@@ -14,9 +14,9 @@ extern int norockridge;
 #endif
 
 #ifdef	_NODOSDRIVE
-# ifdef	NODNAMLEN
+# if	defined (DNAMESIZE) && DNAMESIZE < (MAXNAMLEN + 1)
 typedef struct _st_dirent {
-	char buf[sizeof(struct dirent) + MAXNAMLEN];
+	char buf[sizeof(struct dirent) - DNAMESIZE + MAXNAMLEN + 1];
 } st_dirent;
 # else
 typedef struct dirent	st_dirent;
@@ -228,6 +228,15 @@ int *codep;
 			if (file++ == isrootdir(path)) n++;
 			strncpy2(rpath, path, n);
 		}
+#ifndef	_NODOSDRIVE
+		else if ((n = _dospath(path))) {
+			file = path + 2;
+			rpath[0] = n;
+			rpath[1] = ':';
+			rpath[2] = '.';
+			rpath[3] = '\0';
+		}
+#endif
 		else {
 			file = path;
 			strcpy(rpath, ".");
@@ -448,9 +457,9 @@ char *path;
 {
 	char *cp, conv[MAXPATHLEN];
 
-	if (path == cachecwd || !*cachecwd);
+	if (path == cachecwd || !*cachecwd) /*EMPTY*/;
 #ifndef	_NODOSDRIVE
-	else if (dosdrive && _dospath(cachecwd));
+	else if (dosdrive && _dospath(cachecwd)) /*EMPTY*/;
 #endif
 	else {
 		strcpy(path, cachecwd);
@@ -558,7 +567,8 @@ int bufsiz;
 
 	path = convput(conv, path, 1, 0, lbuf, &c);
 #ifndef	_NOROCKRIDGE
-	if (*lbuf && (len = rrreadlink(lbuf, lbuf, MAXPATHLEN)) >= 0);
+	if (*lbuf && (len = rrreadlink(lbuf, lbuf, MAXPATHLEN)) >= 0)
+		/*EMPTY*/;
 	else
 #endif
 #ifndef	_NODOSDRIVE
