@@ -6,7 +6,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <sys/types.h>
 #include <fcntl.h>
 #include "machine.h"
 
@@ -77,6 +76,7 @@ extern char *realloc2 __P_((VOID_P, ALLOC_T));
 extern char *strncpy3 __P_((char *, char *, int *, int));
 extern int strlen3 __P_((char *));
 extern VOID error __P_((char *));
+extern off_t Xlseek __P_((int, off_t, int));
 
 #if	(!MSDOS && !defined (_NOKANJICONV)) \
 || (!defined (_NOENGMES) && !defined (_NOJPNMES))
@@ -162,57 +162,61 @@ u_char kctypetable[256] = {
 static u_char *unitblbuf = NULL;
 static u_short unitblent = 0;
 static kconv_t rsjistable[] = {
-	{0x8470, 0x8440, 0x0f},		/* strange Russian char */
+	{0x8470, 0x8440, 0x0f},		/* Cyrillic small letters */
 	{0x8480, 0x844f, 0x12},		/* Why they converted ? */
 #define	EXCEPTRUSS	2
-	{0x8754, 0xfa4a, 0x0a},
-	{0x8782, 0xfa59, 0x01},
-	{0x8784, 0xfa5a, 0x01},
-	{0x878a, 0xfa58, 0x01},
-	{0x8790, 0x81e0, 0x01},
-	{0x8791, 0x81df, 0x01},
-	{0x8792, 0x81e7, 0x01},
-	{0x8795, 0x81e3, 0x01},
-	{0x8796, 0x81db, 0x01},
-	{0x8797, 0x81da, 0x01},
-	{0x879a, 0x81e6, 0x01},
-	{0x879b, 0x81bf, 0x01},
-	{0x879c, 0x81be, 0x01},
-	{0xed40, 0xfa5c, 0x23},
-	{0xed63, 0xfa80, 0x1c},
-	{0xed80, 0xfa9c, 0x61},
-	{0xede1, 0xfb40, 0x1c},
-	{0xee40, 0xfb5c, 0x23},
-	{0xee63, 0xfb80, 0x1c},
-	{0xee80, 0xfb9c, 0x61},
-	{0xeee1, 0xfc40, 0x0c},
-	{0xeeef, 0xfa40, 0x0a},
-	{0xeef9, 0x81ca, 0x01},
-	{0xeefa, 0xfa55, 0x03},
-	{0xfa54, 0x81ca, 0x01},
-	{0xfa5b, 0x81e6, 0x01},
+	{0x8754, 0xfa4a, 0x0a},		/* Roman numerals */
+	{0x8782, 0xfa59, 0x01},		/* numero sign */
+	{0x8784, 0xfa5a, 0x01},		/* telephone sign */
+	{0x878a, 0xfa58, 0x01},		/* parenthesized ideograph stock */
+	{0x8790, 0x81e0, 0x01},		/* nearly equals */
+	{0x8791, 0x81df, 0x01},		/* identical to */
+	{0x8792, 0x81e7, 0x01},		/* integral */
+	{0x8795, 0x81e3, 0x01},		/* square root */
+	{0x8796, 0x81db, 0x01},		/* up tack */
+	{0x8797, 0x81da, 0x01},		/* angle */
+	{0x879a, 0x81e6, 0x01},		/* because */
+	{0x879b, 0x81bf, 0x01},		/* intersection */
+	{0x879c, 0x81be, 0x01},		/* union */
+	{0xed40, 0xfa5c, 0x23},		/* NEC-selected IBM extensions */
+	{0xed63, 0xfa80, 0x1c},		/* NEC-selected IBM extensions */
+	{0xed80, 0xfa9c, 0x61},		/* NEC-selected IBM extensions */
+	{0xede1, 0xfb40, 0x1c},		/* NEC-selected IBM extensions */
+	{0xee40, 0xfb5c, 0x23},		/* NEC-selected IBM extensions */
+	{0xee63, 0xfb80, 0x1c},		/* NEC-selected IBM extensions */
+	{0xee80, 0xfb9c, 0x61},		/* NEC-selected IBM extensions */
+	{0xeee1, 0xfc40, 0x0c},		/* NEC-selected IBM extensions */
+	{0xeeef, 0xfa40, 0x0a},		/* small Roman numerals */
+	{0xeef9, 0x81ca, 0x01},		/* full width not sign */
+	{0xeefa, 0xfa55, 0x03},		/* full width broken bar */
+					/* full width apostrophe */
+					/* full width quotation mark */
+	{0xfa54, 0x81ca, 0x01},		/* full width not sign */
+	{0xfa5b, 0x81e6, 0x01},		/* because */
 };
 #define	RSJISTBLSIZ	((int)(sizeof(rsjistable) / sizeof(kconv_t)))
 #endif	/* (!MSDOS && !_NOKANJICONV) || !_NODOSDRIVE */
 #if	!MSDOS && (!defined (_NOKANJICONV) \
 || (!defined (_NODOSDRIVE) && defined (CODEEUC)))
 static kconv_t convtable[] = {
-	{0xfa40, 0xeeef, 0x0a},
-	{0xfa4a, 0x8754, 0x0a},
-	{0xfa54, 0x81ca, 0x01},
-	{0xfa55, 0xeefa, 0x03},
-	{0xfa58, 0x878a, 0x01},
-	{0xfa59, 0x8782, 0x01},
-	{0xfa5a, 0x8784, 0x01},
-	{0xfa5b, 0x81e6, 0x01},
-	{0xfa5c, 0xed40, 0x23},
-	{0xfa80, 0xed63, 0x1c},
-	{0xfa9c, 0xed80, 0x61},
-	{0xfb40, 0xede1, 0x1c},
-	{0xfb5c, 0xee40, 0x23},
-	{0xfb80, 0xee63, 0x1c},
-	{0xfb9c, 0xee80, 0x61},
-	{0xfc40, 0xeee1, 0x0c}
+	{0xfa40, 0xeeef, 0x0a},		/* small Roman numerals */
+	{0xfa4a, 0x8754, 0x0a},		/* Roman numerals */
+	{0xfa54, 0x81ca, 0x01},		/* full width not sign */
+	{0xfa55, 0xeefa, 0x03},		/* full width broken bar */
+					/* full width apostrophe */
+					/* full width quotation mark */
+	{0xfa58, 0x878a, 0x01},		/* parenthesized ideograph stock */
+	{0xfa59, 0x8782, 0x01},		/* numero sign */
+	{0xfa5a, 0x8784, 0x01},		/* telephone sign */
+	{0xfa5b, 0x81e6, 0x01},		/* because */
+	{0xfa5c, 0xed40, 0x23},		/* IBM extensions */
+	{0xfa80, 0xed63, 0x1c},		/* IBM extensions */
+	{0xfa9c, 0xed80, 0x61},		/* IBM extensions */
+	{0xfb40, 0xede1, 0x1c},		/* IBM extensions */
+	{0xfb5c, 0xee40, 0x23},		/* IBM extensions */
+	{0xfb80, 0xee63, 0x1c},		/* IBM extensions */
+	{0xfb9c, 0xee80, 0x61},		/* IBM extensions */
+	{0xfc40, 0xeee1, 0x0c}		/* IBM extensions */
 };
 #define	CNVTBLSIZ	((int)(sizeof(convtable) / sizeof(kconv_t)))
 static u_char sj2jtable1[256] = {
@@ -377,8 +381,8 @@ int ujis2sjis __P_((char *, u_char *, int));
 || !defined (_NODOSDRIVE)
 VOID readunitable __P_((VOID_A));
 VOID discardunitable __P_((VOID_A));
-u_short unifysjis __P_((u_short, int));
-u_short cnvunicode __P_((u_short, int));
+u_int unifysjis __P_((u_int, int));
+u_int cnvunicode __P_((u_int, int));
 #endif
 #if	!MSDOS && !defined (_NOKANJICONV)
 int kanjiconv __P_((char *, char *, int, int, int, int));
@@ -396,24 +400,21 @@ int onkanji1(s, ptr)
 char *s;
 int ptr;
 {
-	int i, j;
+	int i;
 
 	if (ptr < 0) return(0);
 	if (!ptr) return(iskanji1(s, 0));
 
-	for (i = j = 0; i < ptr; i++, j++) {
-		if (!s[j]) return(0);
+	for (i = 0; i < ptr; i++) {
+		if (!s[i]) return(0);
 #ifdef	CODEEUC
-		if (isekana(s, j)) j++;
+		if (isekana(s, i)) i++;
 		else
 #endif
-		if (iskanji1(s, j)) {
-			i++;
-			j++;
-		}
+		if (iskanji1(s, i)) i++;
 	}
 	if (i > ptr) return(0);
-	return(iskanji1(s, j));
+	return(iskanji1(s, i));
 }
 
 #if	(!MSDOS && !defined (_NOKANJICONV)) \
@@ -631,12 +632,13 @@ VOID discardunitable(VOID_A)
 	}
 }
 
-u_short unifysjis(wc, russ)
-u_short wc;
+u_int unifysjis(wc, russ)
+u_int wc;
 int russ;
 {
 	int i;
 
+	wc &= 0xffff;
 	for (i = ((russ) ? 0 : EXCEPTRUSS); i < RSJISTBLSIZ; i++)
 		if (wc >= rsjistable[i].start
 		&& wc < rsjistable[i].start + rsjistable[i].range)
@@ -648,15 +650,17 @@ int russ;
 	return(wc);
 }
 
-u_short cnvunicode(wc, encode)
-u_short wc;
+u_int cnvunicode(wc, encode)
+u_int wc;
 int encode;
 {
 	static int fd = -1;
 	u_char buf[4];
 	char path[MAXPATHLEN];
-	u_short r, w, min, max, ofs;
+	u_short min, max, ofs;
+	u_int r, w;
 
+	wc &= 0xffff;
 	if (encode < 0) {
 		if (fd >= 0) close(fd);
 		fd = -1;
@@ -752,7 +756,7 @@ int encode;
 	}
 
 	if (encode) {
-		if (lseek(fd, (off_t)2, L_SET) < 0) return(r);
+		if (Xlseek(fd, (off_t)2, L_SET) < (off_t)0) return(r);
 		wc = unifysjis(wc, 0);
 		for (ofs = 0; ofs < unitblent; ofs++) {
 			if (read(fd, buf, 4) != 4) break;
@@ -769,7 +773,8 @@ int encode;
 		ofs = unitblent / 2 + 1;
 		for (;;) {
 			if (ofs == min || ofs == max) break;
-			if (lseek(fd, (off_t)(ofs - 1) * 4 + 2, L_SET) < 0
+			if (Xlseek(fd, (off_t)(ofs - 1) * 4 + 2, L_SET)
+			< (off_t)0
 			|| read(fd, buf, 4) != 4) break;
 			w = (((u_short)(buf[1]) << 8) | buf[0]);
 			if (wc == w) {
@@ -1099,7 +1104,7 @@ int max;
 		else if (iseuc(s[i]) && iseuc(s[i + 1])) {
 			u_char tmp[2];
 
-			j2sj(tmp, &(s[i++]));
+			j2sj((char *)tmp, &(s[i++]));
 			w = (tmp[0] << 8) | tmp[1];
 		}
 #  else
@@ -1199,7 +1204,7 @@ int max;
 		else if (iseuc(s[i]) && iseuc(s[i + 1])) {
 			u_char tmp[2];
 
-			j2sj(tmp, &(s[i++]));
+			j2sj((char *)tmp, &(s[i++]));
 			j += bin2hex(&(buf[j]), tmp[0]);
 			j += bin2hex(&(buf[j]), tmp[1]) - 1;
 		}
@@ -1294,7 +1299,7 @@ int max;
 		else if (iseuc(s[i]) && iseuc(s[i + 1])) {
 			u_char tmp[2];
 
-			j2sj(tmp, &(s[i++]));
+			j2sj((char *)tmp, &(s[i++]));
 			j += bin2cap(&(buf[j]), tmp[0]);
 			j += bin2cap(&(buf[j]), tmp[1]) - 1;
 		}
