@@ -27,6 +27,7 @@ extern char *dosgetcwd __P_((char *, int));
 #endif
 
 #ifdef	_NOORIGSHELL
+#include <signal.h>
 #define	dosystem(s)		system(s)
 #else
 #include "system.h"
@@ -42,13 +43,13 @@ extern char **environ;
 extern char fullpath[];
 extern char *origpath;
 extern int hideclock;
-#ifndef	DECLERRLIST
-extern char *sys_errlist[];
-#endif
 #ifndef	_NODOSDRIVE
 extern int lastdrv;
 #endif
 
+#ifndef	SIG_ERR
+#define	SIG_ERR		((sigcst_t)-1)
+#endif
 #ifndef	CHAR_BIT
 # ifdef	NBBY
 # define	CHAR_BIT	NBBY
@@ -61,7 +62,9 @@ extern int lastdrv;
 			| (((u_char *)(cp))[1] << (CHAR_BIT * 2)) \
 			| (((u_char *)(cp))[0] << (CHAR_BIT * 3)) )
 
+#if	!MSDOS
 static int NEAR evallink __P_((char *, char *));
+#endif
 static char *NEAR _realpath2 __P_((char *, char *, int));
 #ifdef	_NOORIGSHELL
 static int NEAR _getenv2 __P_((char *, int, char **));
@@ -100,10 +103,13 @@ struct stat *stp;
 	int duperrno;
 
 	if (Xstat(path, stp) < 0) {
-# ifndef	_NODOSDRIVE
-		if (dospath2(path)) return(-1);
-# endif
 		duperrno = errno;
+# ifndef	_NODOSDRIVE
+		if (dospath2(path)) {
+			errno = duperrno;
+			return(-1);
+		}
+# endif
 		if (nodoslstat(path, stp) < 0
 		|| (stp -> st_mode & S_IFMT) != S_IFLNK) {
 			errno = duperrno;
@@ -121,21 +127,29 @@ char *path, *delim;
 {
 	struct stat st;
 	char buf[MAXPATHLEN];
-	int i;
+	int i, duperrno;
 
-	if (nodoslstat(path, &st) < 0) return(-1);
-	if ((st.st_mode & S_IFMT) != S_IFLNK) return(0);
-	if ((i = Xreadlink(path, buf, MAXPATHLEN - 1)) < 0) return(-1);
-
-	if (*buf == _SC_) strncpy2(path, buf, i);
-	else {
-		if (!delim || delim == path) {
-			*path = _SC_;
-			delim = path + 1;
-		}
-		buf[i] = *delim = '\0';
-		_realpath2(buf, path, 1);
+	duperrno = errno;
+	if (nodoslstat(path, &st) < 0) {
+		errno = duperrno;
+		return(-1);
 	}
+	if ((st.st_mode & S_IFMT) != S_IFLNK) {
+		errno = duperrno;
+		return(0);
+	}
+	if ((i = Xreadlink(path, buf, MAXPATHLEN - 1)) < 0) {
+		errno = duperrno;
+		return(-1);
+	}
+
+	if (*buf == _SC_ || !delim || delim == path) {
+		*path = _SC_;
+		delim = path + 1;
+	}
+	buf[i] = *delim = '\0';
+	_realpath2(buf, path, 1);
+	errno = duperrno;
 	return(1);
 }
 #endif	/* !MSDOS */
@@ -167,7 +181,7 @@ int rdlink;
 		else
 # endif
 		if (rdlink && evallink(resolved, cp) > 0)
-			return(_realpath2(path, resolved, 1));
+			return(_realpath2(path, resolved, rdlink));
 #endif	/* !MSDOS */
 		if (!cp || cp == top) {
 			*top = _SC_;
@@ -264,10 +278,12 @@ char *path;
 	if (Xchdir(path) < 0) return(-1);
 #if	!MSDOS && !defined (_NODOSDRIVE)
 	if (!dospath2("")) {
-		int fd;
+		int fd, duperrno;
 
 		if ((fd = open(".", O_RDONLY, 0600)) < 0) {
+			duperrno = errno;
 			if (Xchdir(cwd) < 0) lostcwd(cwd);
+			errno = duperrno;
 			return(-1);
 		}
 		close(fd);
@@ -280,6 +296,7 @@ int chdir2(path)
 char *path;
 {
 	char *pwd, cwd[MAXPATHLEN], tmp[MAXPATHLEN];
+	int duperrno;
 
 #ifdef	DEBUG
 	if (!path) {
@@ -294,14 +311,14 @@ char *path;
 #endif
 
 	realpath2(path, tmp, physical_path);
-	if (_chdir2(path) < 0) return(-1);
-
 	strcpy(cwd, fullpath);
 	strcpy(fullpath, tmp);
 
 	if (_chdir2(fullpath) < 0) {
+		duperrno = errno;
 		if (_chdir2(cwd) < 0) lostcwd(fullpath);
 		else strcpy(fullpath, cwd);
+		errno = duperrno;
 		return(-1);
 	}
 	if (lastpath) free(lastpath);
@@ -320,7 +337,7 @@ char *path;
 		unixpath = NULL;
 	}
 #endif
-	if (getenv("PWD")) {
+	if (getconstvar("PWD")) {
 #ifdef	DEBUG
 		_mtrace_file = "chdir2(PWD)";
 #endif
@@ -455,10 +472,27 @@ char *strdup2(s)
 char *s;
 {
 	char *tmp;
+	int n;
 
 	if (!s) return(NULL);
-	if (!(tmp = (char *)malloc((ALLOC_T)strlen(s) + 1))) error("malloc()");
-	strcpy(tmp, s);
+	n = strlen(s);
+	if (!(tmp = (char *)malloc((ALLOC_T)n + 1))) error("malloc()");
+	memcpy(tmp, s, n + 1);
+	return(tmp);
+}
+
+char *strndup2(s, n)
+char *s;
+int n;
+{
+	char *tmp;
+	int i;
+
+	if (!s) return(NULL);
+	for (i = 0; i < n; i++) if (!s[i]) break;
+	if (!(tmp = (char *)malloc((ALLOC_T)i + 1))) error("malloc()");
+	memcpy(tmp, s, i);
+	tmp[i] = '\0';
 	return(tmp);
 }
 
@@ -617,6 +651,35 @@ char *s;
 	return((int)n);
 }
 
+#ifdef	USESTDARGH
+/*VARARGS1*/
+int fprintf2(FILE *fp, CONST char *fmt, ...)
+#else
+/*VARARGS1*/
+int fprintf2(fp, fmt, va_alist)
+FILE *s;
+char *fmt;
+va_dcl
+#endif
+{
+	va_list args;
+	char *buf;
+	int n;
+
+#ifdef	USESTDARGH
+	va_start(args, fmt);
+#else
+	va_start(args);
+#endif
+
+	n = vasprintf2(&buf, fmt, args);
+	va_end(args);
+	if (n < 0) error("malloc()");
+	fputs(buf, fp);
+	free(buf);
+	return(n);
+}
+
 #ifdef	USESTDARGH
 /*VARARGS1*/
 int snprintf2(char *s, int size, CONST char *fmt, ...)
@@ -657,11 +720,8 @@ char *s;
 		kanjifputs(s, stderr);
 		fputs(": ", stderr);
 	}
-#if	MSDOS
-	fputs(strerror(duperrno), stderr);
-#else
-	fputs((char *)sys_errlist[duperrno], stderr);
-#endif
+	fputs(strerror2(duperrno), stderr);
+	if (isttyiomode) fputc('\r', stderr);
 	fputc('\n', stderr);
 	fflush(stderr);
 }
@@ -801,6 +861,22 @@ char *name, *value;
 	return(0);
 }
 
+#ifdef	USESIGACTION
+sigcst_t signal2(sig, func)
+int sig;
+sigcst_t func;
+{
+	struct sigaction act, oact;
+
+	act.sa_handler = func;
+	act.sa_flags = 0;
+	sigemptyset(&(act.sa_mask));
+	sigemptyset(&(oact.sa_mask));
+	if (sigaction(sig, &act, &oact) < 0) return(SIG_ERR);
+	return(oact.sa_handler);
+}
+#endif	/* USESIGACTION */
+
 int system2(command, noconf)
 char *command;
 int noconf;
@@ -824,7 +900,8 @@ int noconf;
 	ret = dosystem(command);
 	sigvecset(n);
 	if (ret >= 127 && noconf > 0) {
-		fputs("\007\n", stderr);
+		if (dumbterm <= 2) fputc('\007', stderr);
+		fputc('\n', stderr);
 		kanjifputs(HITKY_K, stderr);
 		fflush(stderr);
 		ttyiomode(1);
@@ -883,7 +960,8 @@ char *command, *type;
 		}
 	}
 	else {
-		fputs("\007\n", stderr);
+		if (dumbterm <= 2) fputc('\007', stderr);
+		fputc('\n', stderr);
 		perror2(command);
 		fflush(stderr);
 		ttyiomode(1);
@@ -974,7 +1052,7 @@ time_t t;
 #endif
 
 #ifndef	NOTZFILEH
-	cp = (char *)getenv("TZ");
+	cp = getconstvar("TZ");
 	if (!cp || !*cp) cp = TZDEFAULT;
 	if (cp[0] == _SC_) strcpy(buf, cp);
 	else strcatdelim2(buf, TZDIR, cp);
