@@ -20,6 +20,8 @@ extern int physical_path;
 #ifdef	_NOORIGSHELL
 static char *NEAR strtkbrk __P_((char *, char *, int));
 static char *NEAR geteostr __P_((char **));
+#else
+#include "system.h"
 #endif
 
 strtable keyidentlist[] = {
@@ -116,7 +118,7 @@ int plus;
 	n = 0;
 	if (plus < 0 && *cp == '-') {
 		for (cp++; isdigit(*cp); cp++) {
-			if (n == MINTYPE(long));
+			if (n == MINTYPE(long)) /*EMPTY*/;
 			else if (n <= MINTYPE(long) / 10
 			&& *cp >= MINTYPE(long) % 10 + '0')
 				n = MINTYPE(long);
@@ -129,7 +131,7 @@ int plus;
 			n = *(cp++) - '0';
 		}
 		for (; isdigit(*cp); cp++) {
-			if (n == MAXTYPE(long));
+			if (n == MAXTYPE(long)) /*EMPTY*/;
 			else if (n >= MAXTYPE(long) / 10
 			&& *cp >= MAXTYPE(long) % 10 + '0')
 				n = MAXTYPE(long);
@@ -151,7 +153,7 @@ int plus;
  */
 char *ascnumeric(buf, n, digit, max)
 char *buf;
-long n;
+off_t n;
 int digit, max;
 {
 	char tmp[MAXLONGWIDTH * 2 + 1];
@@ -191,39 +193,28 @@ char *s, *c;
 int evaldq;
 {
 	char *cp;
-	int quote;
+	int pc, quote;
 
 	for (cp = s, quote = '\0'; *cp; cp++) {
-		if (*cp == quote) {
-			quote = '\0';
-			continue;
-		}
-# ifdef	CODEEUC
-		else if (isekana(cp, 0)) {
-			cp++;
-			continue;
-		}
+# if	MSDOS
+		pc = parsechar(cp, -1, '\0', 0, &quote, NULL);
+# else
+		pc = parsechar(cp, -1, '\0', 1, &quote, NULL);
 # endif
-		else if (iskanji1(cp, 0)) {
+		if (pc == PC_OPQUOTE || pc == PC_CLQUOTE) continue;
+		else if (pc == PC_SQUOTE || pc == PC_BQUOTE) continue;
+		else if (pc == PC_DQUOTE) {
+			if (!evaldq) continue;
+		}
+		else if (pc == PC_WORD) {
 			cp++;
 			continue;
 		}
-		else if (quote == '\'') continue;
-		else if (ismeta(cp, 0, quote)) {
+		else if (pc == PC_META) {
 			cp++;
 			if (*cp == PMETA && strchr(c, *cp)) return(cp - 1);
 			continue;
 		}
-# if	MSDOS
-		else if (quote == '"' && !evaldq) continue;
-		else if (*cp == '\'' || *cp == '"') {
-# else
-		else if (quote == '`' || (quote == '"' && !evaldq)) continue;
-		else if (*cp == '\'' || *cp == '"' || *cp == '`') {
-# endif
-			quote = *cp;
-			continue;
-		}
 
 		if (strchr(c, *cp)) return(cp);
 	}
@@ -251,7 +242,7 @@ char **strp;
 	if ((tmp = strtkbrk(*strp, " \t", 0))) len = tmp - *strp;
 	else len = strlen(*strp);
 	*strp += len;
-	return(strdupcpy(cp, len));
+	return(strndup2(cp, len));
 }
 
 int getargs(s, argvp)
@@ -417,23 +408,22 @@ int delim;
 char *killmeta(name)
 char *name;
 {
-	char *cp, buf[MAXPATHLEN * 2 + 1];
+	char *cp, *buf;
 	int i;
-# ifndef	CODEEUC
-	int sjis;
-
-	cp = (char *)getenv("LANG");
-	sjis = (cp && toupper2(*cp) == 'J' && strchr("AP", toupper2(cp[1])));
-# endif
 
+	buf = malloc2(strlen(name) * 2 + 2 + 1);
 	*buf = (*name == '~') ? '"' : '\0';
 	for (cp = name, i = 1; *cp; cp++, i++) {
-# ifndef	CODEEUC
-		if (sjis && issjis1(cp[0]) && issjis2(cp[1]))
-			buf[i++] = *(cp++);
+# ifdef	CODEEUC
+		if (isekana(cp, 0)) buf[i++] = *(cp++);
 		else
 # endif
-		if (strchr(METACHAR, *cp)) {
+		if (iskanji1(cp, 0)) buf[i++] = *(cp++);
+		else if (*cp == PMETA) {
+			*buf = '"';
+			if (strchr(DQ_METACHAR, *(cp + 1))) buf[i++] = PMETA;
+		}
+		else if (strchr(METACHAR, *cp)) {
 			*buf = '"';
 			if (strchr(DQ_METACHAR, *cp)) buf[i++] = PMETA;
 		}
@@ -442,7 +432,9 @@ char *name;
 	if (*(cp = buf)) buf[i++] = *cp;
 	else cp++;
 	buf[i] = '\0';
-	return(strdup2(cp));
+	cp = strdup2(cp);
+	free(buf);
+	return(cp);
 }
 #endif	/* !MSDOS || !_NOORIGSHELL */
 
@@ -451,7 +443,7 @@ VOID adjustpath(VOID_A)
 {
 	char *cp, *path;
 
-	if (!(cp = (char *)getenv("PATH"))) return;
+	if (!(cp = getconstvar("PATH"))) return;
 
 	path = evalpaths(cp, PATHDELIM);
 	if (strpathcmp(path, cp)) {
@@ -530,14 +522,19 @@ char **bufp, *prompt;
 {
 #if	!MSDOS
 	uidtable *up;
-#endif
-#ifdef	USEUNAME
+# ifdef	USEUNAME
 	struct utsname uts;
+# endif
 #endif
 	char *cp, *tmp, line[MAXPATHLEN];
 	ALLOC_T size;
 	int i, j, k, len, unprint;
 
+#ifdef	_NOORIGSHELL
+	prompt = evalpath(strdup2(prompt), 0);
+#else
+	prompt = evalvararg(prompt, 0, 1, '\0', 0, 0);
+#endif
 	unprint = 0;
 #ifdef	FAKEUNINIT
 	size = 0;	/* fake for -Wuninitialized */
@@ -545,7 +542,7 @@ char **bufp, *prompt;
 	*bufp = c_realloc(NULL, 0, &size);
 	for (i = j = len = 0; prompt[i]; i++) {
 		cp = NULL;
-		*line = '\0';
+		line[0] = '\0';
 		if (prompt[i] != META) {
 			k = 0;
 			line[k++] = prompt[i];
@@ -563,8 +560,7 @@ char **bufp, *prompt;
 				line[1] = '\0';
 				break;
 			case '!':
-				ascnumeric(line, (long)(histno[0]) + 1L,
-					0, MAXPATHLEN - 1);
+				long2str(line, histno[0] + 1, sizeof(line));
 				break;
 #if	!MSDOS
 			case 'u':
@@ -573,12 +569,12 @@ char **bufp, *prompt;
 				break;
 			case 'h':
 			case 'H':
-#ifdef	USEUNAME
+# ifdef	USEUNAME
 				uname(&uts);
 				strcpy(line, uts.nodename);
-#else
+# else
 				gethostname(line, MAXPATHLEN);
-#endif
+# endif
 				if (prompt[i] == 'h'
 				&& (tmp = strchr(line, '.'))) *tmp = '\0';
 				break;
@@ -666,6 +662,7 @@ char **bufp, *prompt;
 		}
 	}
 	(*bufp)[j] = '\0';
+	free(prompt);
 	return(len);
 }
 
@@ -897,7 +894,7 @@ int len;
 		if (isekana(s, i)) cp[j++] = s[i++];
 		else
 #else
-		if (iskna(s[i]));
+		if (iskna(s[i])) /*EMPTY*/;
 		else
 #endif
 		if (iskanji1(s, i)) cp[j++] = s[i++];
